@@ -35,6 +35,7 @@
 #include "kis_canvas_resource_provider.h"
 #include <KisHandlePainterHelper.h>
 #include "KoPathPointTypeCommand.h"
+#include <KisAngleSelector.h>
 
 #include <klocalizedstring.h>
 
@@ -567,11 +568,11 @@ QList<QPointer<QWidget> > KoCreatePathTool::createOptionWidgets()
     angleWidget->setObjectName("Angle Constraints");
     QGridLayout *layout = new QGridLayout(angleWidget);
     layout->addWidget(new QLabel(i18n("Angle snapping delta:"), angleWidget), 0, 0);
-    QSpinBox *angleEdit = new KisIntParseSpinBox(angleWidget);
-    angleEdit->setValue(d->angleSnappingDelta);
+    KisAngleSelector *angleEdit = new KisAngleSelector(angleWidget);
+    angleEdit->setAngle(d->angleSnappingDelta);
     angleEdit->setRange(1, 360);
-    angleEdit->setSingleStep(1);
-    angleEdit->setSuffix(QChar(Qt::Key_degree));
+    angleEdit->setDecimals(0);
+    angleEdit->setFlipOptionsMode(KisAngleSelector::FlipOptionsMode_MenuButton);
     layout->addWidget(angleEdit, 0, 1);
     layout->addWidget(new QLabel(i18n("Activate angle snap:"), angleWidget), 1, 0);
     QCheckBox *angleSnap = new QCheckBox(angleWidget);
@@ -584,7 +585,7 @@ QList<QPointer<QWidget> > KoCreatePathTool::createOptionWidgets()
     angleWidget->setWindowTitle(i18n("Angle Constraints"));
     list.append(angleWidget);
 
-    connect(angleEdit, SIGNAL(valueChanged(int)), this, SLOT(angleDeltaChanged(int)));
+    connect(angleEdit, SIGNAL(angleChanged(qreal)), this, SLOT(angleDeltaChanged(qreal)));
     connect(angleSnap, SIGNAL(stateChanged(int)), this, SLOT(angleSnapChanged(int)));
 
     return list;
