@@ -41,6 +41,7 @@
 #include <kis_debug.h>
 #include <KisImportExportUtils.h>
 #include <kis_config.h>
+#include "kis_scratch_pad.h"
 
 #include "kritaui_export.h"
 
@@ -360,7 +361,7 @@ public:
     void setMirrorAxisConfig(const KisMirrorAxisConfig& config);
 
     QList<KoColorSet *> &paletteList();
-    void setPaletteList(const QList<KoColorSet *> &paletteList);
+    void setPaletteList(const QList<KoColorSet *> &paletteList, bool emitSignal = false);
 
     void clearUndoHistory();
 
@@ -463,6 +464,18 @@ Q_SIGNALS:
 
     void sigMirrorAxisConfigChanged();
 
+    void sigGridConfigChanged(const KisGridConfig &config);
+
+    void sigReferenceImagesLayerChanged(KisSharedPtr<KisReferenceImagesLayer> layer);
+
+    /**
+     * Emitted when the palette list has changed.
+     * The pointers in oldPaletteList are to be deleted by the resource server.
+     **/
+    void sigPaletteListChanged(const QList<KoColorSet *> &oldPaletteList, const QList<KoColorSet *> &newPaletteList);
+
+    void sigAssistantsChanged();
+
 private Q_SLOTS:
     void finishExportInBackground();
     void slotChildCompletedSavingInBackground(KisImportExportErrorCode status, const QString &errorMessage);
@@ -472,6 +485,8 @@ private Q_SLOTS:
 
     void slotInitiateAsyncAutosaving(KisDocument *clonedDocument);
 
+    void slotPerformIdleRoutines();
+
 private:
 
     friend class KisPart;
@@ -625,6 +640,17 @@ public:
      */
     QRectF documentBounds() const;
 
+    /**
+     * @brief Start saving when android activity is pushed to the background
+     */
+    void autoSaveOnPause();
+
+    /**
+      * @brief Helper method to convert a URI to path. Specifically for handling Android's
+      * "content://" URIs
+      */
+    QString toPath(const QUrl& url) const;
+
 Q_SIGNALS:
 
     void completed();
@@ -633,6 +659,7 @@ Q_SIGNALS:
 private Q_SLOTS:
 
     void setImageModified();
+    void setImageModifiedWithoutUndo();
 
     void slotAutoSave();
 
@@ -640,8 +667,8 @@ private Q_SLOTS:
 
     void slotConfigChanged();
 
+    void slotImageRootChanged();
 
-private:
     /**
      * @brief try to clone the image. This method handles all the locking for you. If locking
      *        has failed, no cloning happens
@@ -649,6 +676,21 @@ private:
      */
     KisDocument *lockAndCloneForSaving();
 
+public:
+
+    KisDocument *lockAndCreateSnapshot();
+
+    void copyFromDocument(const KisDocument &rhs);
+
+private:
+
+    enum CopyPolicy {
+        CONSTRUCT = 0, ///< we are copy-constructing a new KisDocument
+        REPLACE ///< we are replacing the current KisDocument with another
+    };
+
+    void copyFromDocumentImpl(const KisDocument &rhs, CopyPolicy policy);
+
     QString exportErrorToUserMessage(KisImportExportErrorCode status, const QString &errorMessage);
 
     QString prettyPathOrUrl() const;
