@@ -31,6 +31,7 @@ KisCmbGradient::KisCmbGradient(QWidget *parent)
     : KisPopupButton(parent)
     , m_gradientChooser(new KisGradientChooser(this))
 {
+    m_gradientChooser->setContentsMargins(10, 10, 10, 10);
     connect(m_gradientChooser, SIGNAL(resourceSelected(KoResource*)), SLOT(gradientSelected(KoResource*)));
     setPopupWidget(m_gradientChooser);
 }
@@ -60,8 +61,11 @@ QSize KisCmbGradient::sizeHint() const
 {
     ensurePolished();
     QFontMetrics fm = fontMetrics();
-
+#if QT_VERSION >= QT_VERSION_CHECK(5,11,0)
+    int maxW = 7 * fm.horizontalAdvance(QChar('x')) + 18;
+#else
     int maxW = 7 * fm.width(QChar('x')) + 18;
+#endif
     int maxH = qMax(fm.lineSpacing(), 14) + 2;
 
     QStyleOptionComboBox options;
