@@ -1,4 +1,7 @@
 #!/usr/bin/env bash
+#
+#  SPDX-License-Identifier: GPL-3.0-or-later
+#
 
 # Krita tool to create dmg from installed source
 # Copies all files to a folder to be converted into the final dmg
@@ -92,14 +95,14 @@ export QMAKE_MACOSX_DEPLOYMENT_TARGET=10.11
 
 
 print_usage () {
-    printf "USAGE: 
+    printf "USAGE:
   osxdeploy.sh [-s=<identity>] [-notarize-ac=<apple-account>] [-style=<style.txt>] [-bg=<background-image>]
 
     -s \t\t\t Code sign identity for codesign
 
     -notarize-ac \t Apple account name for notarization purposes
 \t\t\t script will attempt to get password from keychain, if fails provide one with
-\t\t\t the -notarize-pass option: To add a password run 
+\t\t\t the -notarize-pass option: To add a password run
 
 \t\t\t   security add-generic-password -a \"AC_USERNAME\" -w <secret_password> -s \"AC_PASSWORD\"
 
@@ -428,7 +431,7 @@ macdeployqt_exists() {
         printf "Not Found!\n"
         printf "Attempting to install macdeployqt\n"
 
-        cd ${BUILDROOT}/depbuild/ext_qt/ext_qt-prefix/src/ext_qt/qttools/src
+        cd "${BUILDROOT}/depbuild/ext_qt/ext_qt-prefix/src/ext_qt/qttools/src"
         make sub-macdeployqt-all
         make sub-macdeployqt-install_subtargets
         make install
@@ -506,9 +509,15 @@ krita_deploy () {
     rsync -prul ${KIS_INSTALL_DIR}/translations/ \
             ${KRITA_DMG}/krita.app/Contents/Resources/translations
 
-    echo "Copying kritaquicklook..."
+    echo "Copying QuickLook plugin..."
     mkdir -p ${KRITA_DMG}/krita.app/Contents/Library/QuickLook
     rsync -prul ${KIS_INSTALL_DIR}/plugins/kritaquicklook.qlgenerator ${KRITA_DMG}/krita.app/Contents/Library/QuickLook
+    echo "Copying Spotlight plugin..."
+    mkdir -p ${KRITA_DMG}/krita.app/Contents/Library/Spotlight
+    rsync -prul ${KIS_INSTALL_DIR}/plugins/kritaspotlight.mdimporter ${KRITA_DMG}/krita.app/Contents/Library/Spotlight
+    # TODO fix and reenable - https://bugs.kde.org/show_bug.cgi?id=430553
+    # echo "Copying QuickLook Thumbnailing extension..."
+    # rsync -prul ${KIS_INSTALL_DIR}/plugins/kritaquicklookng.appex ${KRITA_DMG}/krita.app/Contents/PlugIns
 
     cd ${KRITA_DMG}/krita.app/Contents
     ln -shF Resources share
@@ -521,11 +530,12 @@ krita_deploy () {
     cd ${KIS_INSTALL_DIR}/plugins/
     rsync -prul --delete --delete-excluded ./ \
         --exclude kritaquicklook.qlgenerator \
+        --exclude kritaspotlight.mdimporter \
         ${KRITA_DMG}/krita.app/Contents/PlugIns
 
     cd ${BUILDROOT}
     rsync -prul ${KIS_INSTALL_DIR}/lib/kritaplugins/ ${KRITA_DMG}/krita.app/Contents/PlugIns
-    
+
     # rsync -prul {KIS_INSTALL_DIR}/lib/libkrita* Frameworks/
 
     # To avoid errors macdeployqt must be run from bin location
@@ -556,7 +566,7 @@ krita_deploy () {
     rsync -prul ${KIS_INSTALL_DIR}/lib/krita-python-libs ${KRITA_DMG}/krita.app/Contents/Frameworks/
     # change perms on Python to allow header change
     chmod +w ${KRITA_DMG}/krita.app/Contents/Frameworks/Python.framework/Python
-    
+
     fix_python_framework
     strip_python_dmginstall
 
@@ -570,13 +580,18 @@ krita_deploy () {
     rm -rf ${KRITA_DMG}/krita.app/Contents/PlugIns/kf5/org.kde.kwindowsystem.platforms
 
     # Fix permissions
-    find "${KRITA_DMG}/krita.app/Contents" -type f -name "*.dylib" -or -name "*.so" | xargs -P4 -I FILE chmod a+x FILE
+    find "${KRITA_DMG}/krita.app/Contents" -type f -name "*.dylib" -or -name "*.so" -or -path "*/Contents/MacOS/*" | xargs -P4 -I FILE chmod a+x FILE
     find "${KRITA_DMG}/krita.app/Contents/Resources/applications" -name "*.desktop" | xargs -P4 -I FILE chmod a-x FILE
 
     # repair krita for plugins
     printf "Searching for missing libraries\n"
     krita_findmissinglibs $(find ${KRITA_DMG}/krita.app/Contents -type f -perm 755 -or -name "*.dylib" -or -name "*.so")
 
+    # Fix rpath for plugins
+    # Uncomment if the Finder plugins (kritaquicklook, kritaspotlight) lack the rpath below
+    # printf "Repairing rpath for Finder plugins\n"
+    # find "${KRITA_DMG}/krita.app/Contents/Library" -type f -path "*/Contents/MacOS/*" -perm 755 | xargs -I FILE install_name_tool -add_rpath @loader_path/../../../../../Frameworks FILE
+
     printf "removing absolute or broken linksys, if any\n"
     find "${KRITA_DMG}/krita.app/Contents" -type l \( -lname "/*" -or -not -exec test -e {} \; \) -print | xargs rm
 
@@ -601,7 +616,9 @@ signBundle() {
     cd ${KRITA_DMG}/krita.app/Contents/Frameworks
     # remove debug version as both versions can't be signed.
     rm ${KRITA_DMG}/krita.app/Contents/Frameworks/QtScript.framework/Versions/Current/QtScript_debug
-    find . -type f -perm 755 -or -name "*.dylib" -or -name "*.so" | batch_codesign
+    # Do not sign binaries inside frameworks except for Python's
+    find . -type d -path "*.framework" -prune -false -o -perm +111 -not -type d | batch_codesign
+    find Python.framework -type f -name "*.o" -or -name "*.so" -or -perm +111 -not -type d -not -type l | batch_codesign
     find . -type d -name "*.framework" | xargs printf "%s/Versions/Current\n" | batch_codesign
 
     # Sign all other files in Framework (needed)
@@ -616,15 +633,35 @@ signBundle() {
     cd ${KRITA_DMG}/krita.app/Contents/Library/QuickLook
     printf "kritaquicklook.qlgenerator" | batch_codesign
 
-    # It is recommended to sign every Resource file
+    cd ${KRITA_DMG}/krita.app/Contents/Library/Spotlight
+    printf "kritaspotlight.mdimporter" | batch_codesign
+
+    # It is necessary to sign every binary Resource file
     cd ${KRITA_DMG}/krita.app/Contents/Resources
-    find . -type f | batch_codesign
+    find . -perm +111 -type f | batch_codesign
 
     #Finally sign krita and krita.app
     printf "${KRITA_DMG}/krita.app/Contents/MacOS/krita" | batch_codesign
     printf "${KRITA_DMG}/krita.app" | batch_codesign
 }
 
+sign_hasError() {
+    local CODESIGN_STATUS=0
+    for f in $(find "${KRITA_DMG}" -type f); do
+        if [[ -z $(file ${f} | grep "Mach-O") ]]; then
+            continue
+        fi
+
+        CODESIGN_RESULT=$(codesign -vvv --strict ${f} 2>&1 | grep "not signed")
+
+        if [[ -n "${CODESIGN_RESULT}" ]]; then
+            CODESIGN_STATUS=1
+            printf "${f} not signed\n" >&2
+        fi
+    done
+    echo ${CODESIGN_STATUS}
+}
+
 # Notarize build on macOS servers
 # based on https://github.com/Beep6581/RawTherapee/blob/6fa533c40b34dec527f1176d47cc6c683422a73f/tools/osx/macosx_bundle.sh#L225-L250
 notarize_build() {
@@ -653,7 +690,7 @@ notarize_build() {
         local uuid="$(grep 'RequestUUID' <<< ${altoolResponse} | awk '{ print $NF }')"
         echo "RequestUUID = ${uuid}" # Display identifier string
 
-        waiting_fixed "Waiting to retrieve notarize status" 30
+        waiting_fixed "Waiting to retrieve notarize status" 120
 
         while true ; do
             fullstatus=$(xcrun altool --notarization-info "${uuid}" --username "${NOTARIZE_ACC}" --password "${NOTARIZE_PASS}" ${ASC_PROVIDER_OP} 2>&1)  # get the status
@@ -665,7 +702,7 @@ notarize_build() {
                 print_msg "Notarization success!"
                 break
             elif [[ "${notarize_status}" = "in" ]]; then
-                waiting_fixed "Notarization still in progress, sleeping for 20 seconds and trying again" 20
+                waiting_fixed "Notarization still in progress, wait before checking again" 60
             else
                 echo "Notarization failed! full status below"
                 echo "${fullstatus}"
@@ -678,7 +715,7 @@ notarize_build() {
 createDMG () {
     printf "Creating of dmg with contents of %s...\n" "${KRITA_DMG}"
     cd ${BUILDROOT}
-    DMG_size=700
+    DMG_size=1500
 
     if [[ -z "${DMG_NAME}" ]]; then
         # Add git version number
@@ -692,8 +729,8 @@ createDMG () {
 
     # create dmg on local system
     # usage of -fsargs minimze gaps at front of filesystem (reduce size)
-    hdiutil create -srcfolder "${KRITA_DMG}" -volname "${DMG_title}" -fs HFS+ \
-        -fsargs "-c c=64,a=16,e=16" -format UDRW -size ${DMG_size}m krita.temp.dmg
+    hdiutil create -srcfolder "${KRITA_DMG}" -volname "${DMG_title}" -fs APFS \
+        -format UDIF -verbose -size ${DMG_size}m krita.temp.dmg
 
     # Next line is only useful if we have a dmg as a template!
     # previous hdiutil must be uncommented
@@ -719,7 +756,7 @@ createDMG () {
     #Set Icon for DMG
     cp -v "${SCRIPT_SOURCE_DIR}/KritaIcon.icns" "/Volumes/${DMG_title}/.VolumeIcon.icns"
     SetFile -a C "/Volumes/${DMG_title}"
-    
+
     chmod -Rf go-w "/Volumes/${DMG_title}"
 
     # ensure all writing operations to dmg are over
@@ -727,7 +764,7 @@ createDMG () {
 
     hdiutil detach $device
     hdiutil convert "krita.temp.dmg" -format UDZO -imagekey -zlib-level=9 -o krita-out.dmg
-    
+
 
     mv krita-out.dmg ${DMG_NAME}
     echo "moved krita-out.dmg to ${DMG_NAME}"
@@ -753,7 +790,16 @@ if [[ -n "${CODE_SIGNATURE}" ]]; then
     signBundle
 fi
 
-# notarize app
+# Manually check every single Mach-O file for signature status
+print_msg "Checking if all files are signed before sending for notarization..."
+if [[ $(sign_hasError) -eq 1 ]]; then
+    print_error "CodeSign errors cannot send to notarize!"
+    echo "krita.app not sent to notarization, stopping...."
+    exit
+fi
+print_msg "Done! all files appear to be correct."
+
+# notarize apple
 notarize_build "${KRITA_DMG}" krita.app
 
 # Create DMG from files inside ${KRITA_DMG} folder
